#import "PickerViewEx.h"

#define BTN_SHOW 0

//PickerViewExの実装
@implementation PickerViewEx

//アラートの表示
- (void)showAlert:(NSString*)title text:(NSString*)text {
    UIAlertView* alert=[[[UIAlertView alloc] 
        initWithTitle:title message:text delegate:nil 
        cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
    [alert show];
}

//テキストボタンの生成
- (UIButton*)makeButton:(CGRect)rect text:(NSString*)text tag:(int)tag {
    UIButton* button=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [button setFrame:rect];
    [button setTitle:text forState:UIControlStateNormal];
    [button setTag:tag];
    [button addTarget:self action:@selector(clickButton:) 
        forControlEvents:UIControlEventTouchUpInside];
    return button;
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];
    _items=[[NSMutableArray array] retain];
        
    //フォントファミリー名の取得(1)
    NSArray* familyNames=[UIFont familyNames];
    
    //NSArrayオブジェクトのソート(2)
    familyNames=[familyNames sortedArrayUsingSelector:@selector(compare:)];
    
    for (int i=0;i<familyNames.count;i++) {
        //フォント名の取得(3)
        NSArray* fontNames=[UIFont fontNamesForFamilyName:
            [familyNames objectAtIndex:i]];
        
        //追加
        fontNames=[fontNames sortedArrayUsingSelector:@selector(compare:)];
        for (int j=0;j<fontNames.count;j++) {
            [_items addObject:[fontNames objectAtIndex:j]];
        }
    }

    //ピッカービューの生成(4)
    _pickerView=[[UIPickerView alloc] init];
    [_pickerView setFrame:CGRectMake(0,0,320,230)];       
    [_pickerView setBackgroundColor:[UIColor blackColor]];
    [_pickerView setDelegate:self];   
    [_pickerView setShowsSelectionIndicator:YES];                    
    [self.view addSubview:_pickerView];
    _selectIdx=0;

    //ボタンの生成
    UIButton* btnShow=[self makeButton:CGRectMake(0,240,90,40) 
        text:@"表示" tag:BTN_SHOW];
    [self.view addSubview:btnShow];
}

//メモリ解放
- (void)dealloc {
    [_items release];
    [super dealloc];
}

//行数の取得(5)
- (NSUInteger)pickerView:(UIPickerView*)pickerView 
    numberOfRowsInComponent:(NSUInteger)component {
    return _items.count;
}

//行のセルの取得(5)
- (UIView*)pickerView:(UIPickerView*)pickerView viewForRow:(NSInteger)row
    forComponent:(NSInteger)component reusingView:(UIView*)view {
    //セルの生成
    UIView* cell=[[[UIView alloc] 
        initWithFrame:CGRectMake(0,0,280,30)] autorelease];
            
    //ラベルの生成
    NSString* fontName=[_items objectAtIndex:row];
    UILabel* label=[[[UILabel alloc] init] autorelease];         
    [label setFrame:CGRectMake(0,0,280,40)];
    [label setFont:[UIFont fontWithName:fontName size:18]];
    [label setBackgroundColor:[UIColor clearColor]];
    [label setText:fontName];   
    [cell addSubview:label];
    return cell;
}

//行の高さの取得(5)
- (CGFloat)pickerView:(UIPickerView*)pickerView 
    rowHeightForComponent:(NSInteger)component {
    return 30;
}

//ピッカー選択時に呼ばれる(5)
- (void)pickerView:(UIPickerView *)pickerView 
    didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
    _selectIdx=row;
}

//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    if (sender.tag==BTN_SHOW) {
        NSString* name=[_items objectAtIndex:_selectIdx];
        [self showAlert:@"" text:[NSString stringWithFormat:
            @"ピッカーの選択\n%@",name]];
    }
}
@end