#import "ICloudEx.h"
#import "ICloudDocument.h"

#define BTN_WRITE0 0
#define BTN_READ0  1
#define BTN_WRITE1 2
#define BTN_READ1  3

//ICloudExの実装
@implementation ICloudEx

//テキストフィールドの生成
- (UITextField*)makeTextField:(CGRect)rect text:(NSString*)text {
    UITextField* textField=[[[UITextField alloc] init] autorelease];
    [textField setText:text];
    [textField setFrame:rect];
    [textField setReturnKeyType:UIReturnKeyDone];
    [textField setBackgroundColor:[UIColor whiteColor]];
    [textField setBorderStyle:UITextBorderStyleRoundedRect];
    [textField setDelegate:self];
    return textField;
}

//テキストボタンの生成
- (UIButton*)makeButton:(CGRect)rect text:(NSString*)text tag:(int)tag {
    UIButton* button=[UIButton buttonWithType:UIButtonTypeRoundedRect];
    [button setFrame:rect];
    [button setTitle:text forState:UIControlStateNormal];
    [button setTag:tag];
    [button addTarget:self action:@selector(clickButton:) 
        forControlEvents:UIControlEventTouchUpInside];
    return button;
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];
    
    //キーバリューのUIの生成
    _textField0=[[self makeTextField:CGRectMake(0,0,300,32) 
        text:@""] retain];
    [self.view addSubview:_textField0];
    UIButton* btnWrite0=[self makeButton:CGRectMake(0,50,190,40) 
        text:@"キーバリューの書き込み" tag:BTN_WRITE0];
    [self.view addSubview:btnWrite0];
    UIButton* btnRead0=[self makeButton:CGRectMake(200,50,90,40) 
        text:@"読み込み" tag:BTN_READ0];
    [self.view addSubview:btnRead0];

    //ドキュメントのUIの生成
    _textField1=[[self makeTextField:CGRectMake(0,120,300,32) 
        text:@""] retain];
    [self.view addSubview:_textField1];
    UIButton* btnWrite1=[self makeButton:CGRectMake(0,170,190,40) 
        text:@"ドキュメントの書き込み" tag:BTN_WRITE1];
    [self.view addSubview:btnWrite1];
    UIButton* btnRead1=[self makeButton:CGRectMake(200,170,90,40) 
        text:@"読み込み" tag:BTN_READ1];
    [self.view addSubview:btnRead1];

}

//メモリ解放
- (void)dealloc {
    [_textField0 release];
    [_textField1 release];
    [super dealloc];
}

//テキストフィールドのリターン時に呼ばれる
- (BOOL)textFieldShouldReturn:(UITextField*)sender {
    [sender resignFirstResponder];
    return YES;
}

//iCloudのドキュメントのURLの生成(3)
- (NSURL*)makeICloudURL:(NSString*)fileName {
    //iCloudのディレクトリのURLの生成
    NSFileManager* fileManager=[NSFileManager defaultManager];
    NSURL* icloudURL=[fileManager URLForUbiquityContainerIdentifier:
        @"7T4792K37P.net.npaka.icloudex"];
    icloudURL=[icloudURL URLByAppendingPathComponent:@"Documents"];
    
    //ディレクトリがない時は生成
    if ([fileManager fileExistsAtPath:[icloudURL path]]==NO) {
        [fileManager createDirectoryAtURL:icloudURL 
            withIntermediateDirectories:YES attributes:nil error:nil];
    }
    
    //iCloudファイルのURL
    icloudURL=[icloudURL URLByAppendingPathComponent:fileName];    
    return icloudURL;
}

//ボタンクリック時に呼ばれる
- (IBAction)clickButton:(UIButton*)sender {
    if (sender.tag==BTN_WRITE0) {
        //iCloudへのキーバリューの書き込み(1)
        NSUbiquitousKeyValueStore* store=
            [NSUbiquitousKeyValueStore defaultStore];
        [store setObject:_textField0.text forKey:@"text"];
        [store synchronize];
    } else if (sender.tag==BTN_READ0) {
        //iCloudからのキーバリューの読み込み(2)
        NSUbiquitousKeyValueStore* store=
            [NSUbiquitousKeyValueStore defaultStore];
        _textField0.text=[store stringForKey:@"text"];
    } else if (sender.tag==BTN_WRITE1) {
        //iCloudへのドキュメントの書き込み(3)
        NSURL* icloudURL=[self makeICloudURL:@"test.txt"];
        ICloudDocument* document=[[[ICloudDocument alloc] 
             initWithFileURL:icloudURL] autorelease];
        document.text=_textField1.text;
        [document saveToURL:icloudURL forSaveOperation:
            UIDocumentSaveForCreating completionHandler:^(BOOL success){
            NSLog(@"write document>%@",success?@"success":@"fail");
        }];
    } else if (sender.tag==BTN_READ1) {
        //iCloudからのドキュメントの読み込み(4)
        NSURL* icloudURL=[self makeICloudURL:@"test.txt"];
        ICloudDocument* document=[[[ICloudDocument alloc] 
            initWithFileURL:icloudURL] autorelease];
        [document openWithCompletionHandler:^(BOOL success){
            NSLog(@"read document>%@",success?@"success":@"fail");
            _textField1.text=document.text;
        }];
    }
}
@end