#import "AddressEx.h"
#import <AddressBook/AddressBook.h>

//AddressExの実装
@implementation AddressEx

//URLスキームのオープン(1)
- (BOOL)openURL:(NSString*)url {
    return [[UIApplication sharedApplication] openURL:
        [NSURL URLWithString:url]];
}

//アラートの表示
- (void)showAlert:(NSString*)title text:(NSString*)text {
    UIAlertView* alert=[[[UIAlertView alloc] 
        initWithTitle:title message:text delegate:nil 
        cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
    [alert show];
}

//アドレスの読み込み
- (void)readAddress {
    [_names removeAllObjects];
    [_tels removeAllObjects];

    //アドレスのレコードの取得(2)
    ABAddressBookRef book=ABAddressBookCreateWithOptions(NULL,nil);
    CFArrayRef records=ABAddressBookCopyArrayOfAllPeople(book);
    for (int i=0;i<CFArrayGetCount(records);i++) {
        ABRecordRef record=CFArrayGetValueAtIndex(records,i);	
	
        //レコードからの名前の取得(3)
        NSString* firstName=(NSString*)ABRecordCopyValue(
            record,kABPersonFirstNameProperty);
        NSString* lastName =(NSString*)ABRecordCopyValue(
            record,kABPersonLastNameProperty);
        if (firstName==nil) firstName=@"";
        if (lastName ==nil) lastName =@"";
        NSString* name=[NSString stringWithFormat:@"%@ %@",lastName,firstName];
        [_names addObject:name];
        if (firstName!=nil) CFRelease(firstName);
        if (lastName !=nil) CFRelease(lastName);

        //レコードからの電話番号の取得(4)
        ABMultiValueRef tels=ABRecordCopyValue(record,kABPersonPhoneProperty);
        if (ABMultiValueGetCount(tels)>0) {
            NSString* tel=(NSString*)ABMultiValueCopyValueAtIndex(tels,0);
            [_tels addObject:tel];
            CFRelease(tel);	
        } else {
            [_tels addObject:@""];
        }
        CFRelease(tels);
    }
    CFRelease(book);	
    CFRelease(records);
    
    //テーブルビューの更新(5)
    [_tableView reloadData];
}

//ロード完了時に呼ばれる
- (void)viewDidLoad {
    [super viewDidLoad];

    //テーブルビューの生成
    _tableView=[[UITableView alloc] init];
    [_tableView setFrame:self.view.frame];
    [_tableView setDelegate:self];                     
    [_tableView setDataSource:self];
    [self.view addSubview:_tableView];

    //ビューのサイズの自動調整
    _tableView.autoresizingMask= 
         UIViewAutoresizingFlexibleRightMargin| 
         UIViewAutoresizingFlexibleTopMargin|
         UIViewAutoresizingFlexibleLeftMargin|
         UIViewAutoresizingFlexibleBottomMargin|
         UIViewAutoresizingFlexibleWidth|
         UIViewAutoresizingFlexibleHeight;

    //要素
    _names=[[NSMutableArray array] retain];
    _tels =[[NSMutableArray array] retain];
    [self readAddress];
}

//メモリ解放
-(void)dealloc {
    [_tableView release];
    [_names release];
    [_tels release];
    [super dealloc];
}

//セルの高さの取得
- (CGFloat)tableView:(UITableView*)tableView 
    heightForRowAtIndexPath:(NSIndexPath*)indexPath {
    return 50;
}

//セルのクリック時に呼ばれる
- (void)tableView:(UITableView*)tableView 
    didSelectRowAtIndexPath:(NSIndexPath*)indexPath {
    //選択解除
    [_tableView deselectRowAtIndexPath:[
        _tableView indexPathForSelectedRow] animated:YES]; 

    //通話
    NSString* tel=[_tels objectAtIndex:indexPath.row];
    if (tel.length>0) {
        if (![self openURL:[NSString stringWithFormat:@"tel:%@",tel]]) {
            [self showAlert:@"" text:@"通話機能がありません"];
        }
    }
}

//セルの数取得時に呼ばれる
- (NSInteger)tableView:(UITableView*)tableView 
    numberOfRowsInSection:(NSInteger)section {
    return _names.count;
}

//セルの取得時に呼ばれる
- (UITableViewCell*)tableView:(UITableView*)tableView 
    cellForRowAtIndexPath:(NSIndexPath*)indexPath {
    //テーブルのセルの生成
    UITableViewCell* cell=[tableView 
        dequeueReusableCellWithIdentifier:@"table_cell"];
    if (cell==nil) {
        cell=[[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle 
            reuseIdentifier:@"table_cell"] autorelease];
    }
    [cell.textLabel setText:[_names objectAtIndex:indexPath.row]];
    [cell.detailTextLabel setText:[_tels objectAtIndex:indexPath.row]];
    return cell;
}
@end
