#import "GyroEx.h"

@implementation GyroEx
//ラベルのリサイズ
- (void)resizeLabel:(UILabel*)label {
    CGRect frame=label.frame;
    frame.size=[label.text sizeWithFont:label.font 
        constrainedToSize:CGSizeMake(512,512)
        lineBreakMode:NSLineBreakByWordWrapping];
    [label setFrame:frame];
}

//ラベルの生成
- (UILabel*)makeLabel:(CGPoint)pos 
    text:(NSString*)text font:(UIFont*)font {
    UILabel* label=[[[UILabel alloc] init] autorelease];         
    [label setText:text];
    [label setFont:font];
    [label setTextColor:[UIColor blackColor]];
    [label setBackgroundColor:[UIColor clearColor]];
    [label setTextAlignment:NSTextAlignmentLeft];
    [label setNumberOfLines:0];
    [label setLineBreakMode:NSLineBreakByWordWrapping];
    [self resizeLabel:label];
    return label;
}

//回転角度更新時に呼ばれる
- (void)updateGyro:(CMDeviceMotion*)motion {
    //回転角度の取得(2)
    CMAttitude* attitude=motion.attitude;
    double pitch=attitude.pitch*180/M_PI;
    double yaw  =attitude.yaw  *180/M_PI;
    double roll =attitude.roll *180/M_PI;
    
    //ラベルの更新
    NSMutableString* str=[NSMutableString string];
    [str appendString:@"GyroEx\n"];
    [str appendFormat:@"X軸回転角度:%+.0f\n",pitch];
    [str appendFormat:@"Y軸回転角度:%+.0f\n",yaw];
    [str appendFormat:@"Z軸回転角度:%+.0f\n",roll];
    [_label setText:str];
    [self resizeLabel:_label];
}

//初期化
- (void)viewDidLoad {
    [super viewDidLoad];

    //ラベルの生成
    _label=[[self makeLabel:CGPointMake(0,0) 
        text:@"GyroEx" 
        font:[UIFont systemFontOfSize:16]] retain];
    [self.view addSubview:_label];

    //回転角度通知の開始(1)
    _motionManager=[[CMMotionManager alloc] init]; 
    if (_motionManager.gyroAvailable) {
        _motionManager.gyroUpdateInterval=0.1f;
        [_motionManager startDeviceMotionUpdatesToQueue:
            [NSOperationQueue currentQueue]
            withHandler:^(CMDeviceMotion* motion,NSError* error) {
                [self updateGyro:motion];
            }];
    }
}

//メモリの解放
- (void)dealloc {
    [_label release];
    [_motionManager release];
    [super dealloc];
}
@end
