#import "ActionView.h"

//ActionViewの実装
@implementation ActionView

//初期化
- (id)initWithCoder:(NSCoder*)coder {
    self=[super initWithCoder:coder];
    if (self) {
        self.backgroundColor=[UIColor blackColor];
        //システム
        CGRect rect=[[UIScreen mainScreen] bounds];
        int screenW=MAX(rect.size.width,rect.size.height);
        int screenH=MIN(rect.size.width,rect.size.height);
        _g=[[Graphics alloc] init];
        _gScale=screenH/360.0f;
        _gX=((screenW/_gScale)-640)/2;
        _gY=0;
        
        //ゲーム画面のスケールの指定(5)
        [self setTransform:CGAffineTransformMakeScale(_gScale,_gScale)];
       
        //ゲーム
        _scene=0;
        _init =S_TITLE;
        _image=[[NSMutableArray array] retain];
        for (int i=0;i<12;i++) {
            [_image addObject:[UIImage imageNamed:
                [NSString stringWithFormat:@"pic%d.png",i]]];
        }
        _score    =0;
        _touchDown=NO;
        
        //プレイヤー
        _playerX  =100;
        _playerY  =0;
        _playerDY =0;
        _playerHit=NO;
        
        //隕石
        _meteoAppear=0;
        
        //タイマーの開始
        [self onTick:nil];
        [NSTimer scheduledTimerWithTimeInterval:0.05f
            target:self selector:@selector(onTick:)
            userInfo:nil repeats:YES];
    }
    return self;
}

//メモリ解放
- (void)dealloc {
    [_g release];
    [_image release];
    [super dealloc];
}

//定期処理(2)
- (void)onTick:(NSTimer*)timer {
    //シーンの初期化
    if (_init>=0) {
        //タイトルの初期化
        if (_init==S_TITLE) {
            _playerY=130;
            _playerDY=0;
            _playerHit=NO;
            _meteoAppear=0;
            for (int i=0;i<METEO_NUM;i++) _meteoY[i]=-1;
        }
        //プレイの初期化
        else if (_init==S_PLAY) {
            _score=0;
        }
        _touchDown=NO;
        _scene=_init;
        _init=-1;
    }
    
    //プレイ時の処理
    if (_scene==S_PLAY) {
        //プレイヤーの移動(3)
        if (_touchDown && !_playerHit) {
            _playerDY--;
        } else {
            _playerDY++;
        }
        _playerY+=_playerDY;
        if (_playerY<-47 || 360+47<_playerY) _init=S_GAMEOVER;
        
        if (!_playerHit) {
            //隕石の移動と出現(4)
            for (int i=0;i<METEO_NUM;i++) {
                if (_meteoY[i]>=0) {
                    _meteoX[i]-=3;
                    if (_meteoX[i]<-30) _meteoY[i]=-1;
                    if ([self isHit:i]) {
                        _playerHit=YES;
                        _playerDY=5;
                    }
                } else if (_meteoAppear<=0) {
                    _meteoAppear=[self rand:30];
                    _meteoX[i]=640+30;
                    _meteoY[i]=[self rand:360];
                }
            }
            _meteoAppear--;
            
            //スコア加算
            _score++;
        }
    }
    
    //画面の更新    
    [self setNeedsDisplay];
}

//描画(2)
- (void)drawRect:(CGRect)rect {
    //グラフィックスコンテキストの指定
    [_g setContext:UIGraphicsGetCurrentContext()];
    
    //ゲーム画面の原点とスケールの指定(5)
    [_g setTranslate_x:_gX y:_gY];
    
    //背景の描画
    [_g drawImage:_image[4] x:0 y:0];
    if (_scene==S_GAMEOVER) {
        [_g drawImage:_image[8] x:250 y:110];
    }
    
    //プレイヤーと隕石の描画
    int playerIdx=(_playerDY>10)?2:1;
    if (_touchDown && !_playerHit) playerIdx=0;
    [_g drawImage:_image[playerIdx] x:_playerX-22 y:_playerY-47];
    for (int i=0;i<METEO_NUM;i++) {
        if (_meteoY[i]>=0) {
            [_g drawImage:_image[3] x:_meteoX[i]-30 y:_meteoY[i]-30];
        }
    }
    
    //タイトルとボタンの描画
    if (_scene==S_TITLE) {
        [_g drawImage:_image[5] x:125 y:20];
        [_g drawImage:_image[6] x:250 y:260];
    } else if (_scene==S_GAMEOVER) {
        [_g drawImage:_image[7] x:105 y:20];
        [_g drawImage:_image[9] x:250 y:260];
    }
    
    //スコアの描画
    [self drawScore:_score];
}

//スコアの描画
- (void)drawScore:(int)score {
    if (_score>999999) _score=999999;
    int x=640-205+_gX;
    [_g drawImage:_image[10] x:x y:2];
    x+=80;
    int value=1000000;
    for (int i=0;i<6;i++) {
        value/=10;
        int num=score/value;
        [_g drawImage:_image[11] x:x y:2 sx:num*20 sy:0 sw:20 sh:25];
        score-=num*value;
        x+=20;
    }
}

//隕石との衝突判定
- (BOOL)isHit:(int)i {
    return ((_playerX-_meteoX[i])*(_playerX-_meteoX[i])+
        (_playerY-_meteoY[i])*(_playerY-_meteoY[i]))<45*45;
}

//乱数の取得
- (int)rand:(int)num {
    return abs(arc4random())%num;
}

//タッチ開始イベントの処理
- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event {
    _touchDown=YES;
    if (_scene==S_TITLE)    _init=S_PLAY;
    if (_scene==S_GAMEOVER) _init=S_TITLE;
}

//タッチ終了イベントの処理
- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event {
    _touchDown=NO;
}

//タッチキャンセルイベントの処理
- (void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event {
    [self touchesEnded:touches withEvent:event];
}
@end
