#import "Graphics.h"

//Graphicsの実装
@implementation Graphics

//====================
//初期化
//====================
//初期化
- (id)init {
    if (self=[super init]) {
        _context=NULL;
        _font=[UIFont boldSystemFontOfSize:12];
    }
    return self;
}

//メモリ解放
- (void)dealloc {
    [self releaseContext];
    [super dealloc]; 
}

//====================
//コンテキスト設定
//====================
//コンテキストの指定
- (void)setContext:(CGContextRef)context {
    //コンテキストの解放
    [self releaseContext];
    
    //コンテキストの指定
    _context=context;
    CGContextRetain(_context);
}

//コンテキストの解放
- (void)releaseContext {
    if (_context!=NULL) {
        CGContextRelease(_context);
        _context=NULL;
    }
}

//====================
//描画設定
//====================    
//色の指定
- (void)setColor_r:(float)r g:(float)g b:(float)b {
    CGContextSetRGBFillColor(_context,r,g,b,1);
    CGContextSetRGBStrokeColor(_context,r,g,b,1);
}

//色の指定
- (void)setColor_r:(float)r g:(float)g b:(float)b a:(float)a {
    CGContextSetRGBFillColor(_context,r,g,b,a);
    CGContextSetRGBStrokeColor(_context,r,g,b,a);
}

//フォントの指定
- (void)setFont:(UIFont*)font {
    _font=font;
}

//ライン幅の指定
- (void)setLineWidth:(float)lineWidth {
    CGContextSetLineWidth(_context,lineWidth);
}

//文字列の長さの取得
- (int)stringWidth:(NSString*)str {
    return [str sizeWithFont:_font].width;  
}

//ユーザー座標の原点の指定
- (void)setTranslate_x:(float)x y:(float)y {
    CGContextTranslateCTM(_context,x,y);
}

//スケールの指定　
- (void)setScale_w:(float)w h:(float)h {
    CGContextScaleCTM(_context,w,h);
}

//====================
//描画
//====================
//ラインの描画
- (void)drawLine_x0:(float)x0 y0:(float)y0 x1:(float)x1 y1:(float)y1 {
    CGContextSetLineCap(_context,kCGLineCapRound);
    CGContextMoveToPoint(_context,x0,y0);
    CGContextAddLineToPoint(_context,x1,y1);
    CGContextStrokePath(_context);
}

//矩形の描画
- (void)drawRect_x:(float)x y:(float)y w:(float)w h:(float)h {
    CGContextMoveToPoint(_context,x,y);
    CGContextAddLineToPoint(_context,x+w,y);
    CGContextAddLineToPoint(_context,x+w,y+h);
    CGContextAddLineToPoint(_context,x,y+h);
    CGContextAddLineToPoint(_context,x,y);
    CGContextAddLineToPoint(_context,x+w,y);
    CGContextStrokePath(_context);
}

//矩形の塗り潰し
- (void)fillRect_x:(float)x y:(float)y w:(float)w h:(float)h {
    CGContextFillRect(_context,CGRectMake(x,y,w,h));
}

//イメージの描画
- (void)drawImage:(UIImage*)image x:(float)x y:(float)y {
    if (image==nil) return;
    [image drawAtPoint:(CGPointMake(x,y))];
}

//イメージの描画
- (void)drawImage:(UIImage*)image x:(float)x y:(float)y w:(float)w h:(float)h {
    if (image==nil) return;
    [image drawInRect:(CGRectMake(x,y,w,h))];
}

//イメージの描画
- (void)drawImage:(UIImage*)image x:(float)x y:(float)y
    sx:(float)sx sy:(float)sy sw:(float)sw sh:(float)sh {
    if (image==nil) return;
    CGContextSaveGState(_context);
    CGContextClipToRect(_context,CGRectMake(x,y,sw,sh));
    [image drawInRect:(CGRectMake(x-sx,y-sy,image.size.width,image.size.height))];
    CGContextRestoreGState(_context);
}

//文字列の描画
- (void)drawString:(NSString*)string x:(float)x y:(float)y {
    [string drawAtPoint:CGPointMake(x,y) withFont:_font];
}
@end
